# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class VoiceParams(typing_extensions.TypedDict):
    id: str
    """
    The voice ID.
    """

    name: str
    """
    The voice name.
    """

    description: typing.Optional[str]
    """
    The voice description.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .tool_parameter_item_type import ToolParameterItemType
from .tool_parameter_location import ToolParameterLocation
from .tool_parameter_type import ToolParameterType


class ToolParameter(UniversalBaseModel):
    type: ToolParameterType = pydantic.Field()
    """
    The parameter type.
    """

    item_type: typing.Optional[ToolParameterItemType] = pydantic.Field(default=None)
    """
    Required only when type is "array". The type of items in the array.
    """

    name: str = pydantic.Field()
    """
    The parameter name.
    """

    description: str = pydantic.Field()
    """
    Description of the parameter.
    """

    is_required: bool = pydantic.Field()
    """
    Whether the parameter is required.
    """

    location: typing.Optional[ToolParameterLocation] = pydantic.Field(default=None)
    """
    Only applicable for `custom_webhook` tools. Specifies where the parameter should be sent in the webhook request.
    - For GET webhooks: defaults to `"query_string"` and `"request_body"` is not allowed.
    - For POST webhooks: required, can be either `"request_body"` or `"query_string"`.
    - Not allowed for `custom_websocket` or `built_in_transfer_to_phone_number` tools.
    When updating a tool's type or endpoint_method, all parameters must include explicit `location` values.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .agent import Agent
    from .agent_audio_format import AgentAudioFormat
    from .agent_background_noise import AgentBackgroundNoise
    from .agent_configuration_endpoint import AgentConfigurationEndpoint
    from .agent_project import AgentProject
    from .agent_template_variables_value import AgentTemplateVariablesValue
    from .agent_tools_item import AgentToolsItem
    from .assistant_chose_not_to_respond_payload import AssistantChoseNotToRespondPayload
    from .assistant_ended_conversation_payload import AssistantEndedConversationPayload
    from .audio_chunk_payload import AudioChunkPayload
    from .audio_chunk_response_payload import AudioChunkResponsePayload
    from .bad_request_error_body import BadRequestErrorBody
    from .basic_error import BasicError
    from .basic_error_error import BasicErrorError
    from .config_payload import ConfigPayload
    from .config_payload_background_noise import ConfigPayloadBackgroundNoise
    from .config_payload_input_format import ConfigPayloadInputFormat
    from .config_payload_output_format import ConfigPayloadOutputFormat
    from .conversation import Conversation
    from .conversation_agent import ConversationAgent
    from .conversation_analysis import ConversationAnalysis
    from .conversation_background_noise import ConversationBackgroundNoise
    from .conversation_created_payload import ConversationCreatedPayload
    from .conversation_ended_by import ConversationEndedBy
    from .conversation_eval import ConversationEval
    from .conversation_eval_prompt import ConversationEvalPrompt
    from .conversation_evaluation import ConversationEvaluation
    from .conversation_evaluation_result import ConversationEvaluationResult
    from .conversation_evaluation_result_prompt import ConversationEvaluationResultPrompt
    from .conversation_evaluation_result_result import ConversationEvaluationResultResult
    from .conversation_extraction import ConversationExtraction
    from .conversation_extraction_schema import ConversationExtractionSchema
    from .conversation_item import ConversationItem
    from .conversation_item_role import ConversationItemRole
    from .conversation_item_tool_calls_item import ConversationItemToolCallsItem
    from .conversation_item_tool_calls_item_tool import ConversationItemToolCallsItemTool
    from .conversation_project import ConversationProject
    from .create_agent_request import CreateAgentRequest
    from .create_agent_request_audio_format import CreateAgentRequestAudioFormat
    from .create_agent_request_background_noise import CreateAgentRequestBackgroundNoise
    from .create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpoint
    from .create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValue
    from .create_agent_request_tools_item import CreateAgentRequestToolsItem
    from .dtmf_payload import DtmfPayload
    from .error import Error
    from .error_payload import ErrorPayload
    from .error_payload_error import ErrorPayloadError
    from .extraction_field import ExtractionField
    from .extraction_field_type import ExtractionFieldType
    from .extraction_schema import ExtractionSchema
    from .input_cancelled_payload import InputCancelledPayload
    from .input_text_payload import InputTextPayload
    from .is_user_speaking_payload import IsUserSpeakingPayload
    from .language_code import LanguageCode
    from .outbound_call_config import OutboundCallConfig
    from .outbound_call_config_tools_item import OutboundCallConfigToolsItem
    from .project import Project
    from .project_default_agent import ProjectDefaultAgent
    from .ready_to_start_conversation_payload import ReadyToStartConversationPayload
    from .set_external_id_payload import SetExternalIdPayload
    from .set_twilio_call_sid_payload import SetTwilioCallSidPayload
    from .task import Task
    from .tool import Tool
    from .tool_call_interrupted_payload import ToolCallInterruptedPayload
    from .tool_call_output_payload import ToolCallOutputPayload
    from .tool_call_output_processed_payload import ToolCallOutputProcessedPayload
    from .tool_call_output_processed_payload_tool import ToolCallOutputProcessedPayloadTool
    from .tool_call_payload import ToolCallPayload
    from .tool_endpoint_method import ToolEndpointMethod
    from .tool_execution_mode import ToolExecutionMode
    from .tool_parameter import ToolParameter
    from .tool_parameter_item_type import ToolParameterItemType
    from .tool_parameter_location import ToolParameterLocation
    from .tool_parameter_type import ToolParameterType
    from .tool_project import ToolProject
    from .tool_type import ToolType
    from .update_system_prompt_payload import UpdateSystemPromptPayload
    from .user_finished_speaking_payload import UserFinishedSpeakingPayload
    from .user_started_speaking_payload import UserStartedSpeakingPayload
    from .validation_error import ValidationError
    from .validation_error_error import ValidationErrorError
    from .voice import Voice
_dynamic_imports: typing.Dict[str, str] = {
    "Agent": ".agent",
    "AgentAudioFormat": ".agent_audio_format",
    "AgentBackgroundNoise": ".agent_background_noise",
    "AgentConfigurationEndpoint": ".agent_configuration_endpoint",
    "AgentProject": ".agent_project",
    "AgentTemplateVariablesValue": ".agent_template_variables_value",
    "AgentToolsItem": ".agent_tools_item",
    "AssistantChoseNotToRespondPayload": ".assistant_chose_not_to_respond_payload",
    "AssistantEndedConversationPayload": ".assistant_ended_conversation_payload",
    "AudioChunkPayload": ".audio_chunk_payload",
    "AudioChunkResponsePayload": ".audio_chunk_response_payload",
    "BadRequestErrorBody": ".bad_request_error_body",
    "BasicError": ".basic_error",
    "BasicErrorError": ".basic_error_error",
    "ConfigPayload": ".config_payload",
    "ConfigPayloadBackgroundNoise": ".config_payload_background_noise",
    "ConfigPayloadInputFormat": ".config_payload_input_format",
    "ConfigPayloadOutputFormat": ".config_payload_output_format",
    "Conversation": ".conversation",
    "ConversationAgent": ".conversation_agent",
    "ConversationAnalysis": ".conversation_analysis",
    "ConversationBackgroundNoise": ".conversation_background_noise",
    "ConversationCreatedPayload": ".conversation_created_payload",
    "ConversationEndedBy": ".conversation_ended_by",
    "ConversationEval": ".conversation_eval",
    "ConversationEvalPrompt": ".conversation_eval_prompt",
    "ConversationEvaluation": ".conversation_evaluation",
    "ConversationEvaluationResult": ".conversation_evaluation_result",
    "ConversationEvaluationResultPrompt": ".conversation_evaluation_result_prompt",
    "ConversationEvaluationResultResult": ".conversation_evaluation_result_result",
    "ConversationExtraction": ".conversation_extraction",
    "ConversationExtractionSchema": ".conversation_extraction_schema",
    "ConversationItem": ".conversation_item",
    "ConversationItemRole": ".conversation_item_role",
    "ConversationItemToolCallsItem": ".conversation_item_tool_calls_item",
    "ConversationItemToolCallsItemTool": ".conversation_item_tool_calls_item_tool",
    "ConversationProject": ".conversation_project",
    "CreateAgentRequest": ".create_agent_request",
    "CreateAgentRequestAudioFormat": ".create_agent_request_audio_format",
    "CreateAgentRequestBackgroundNoise": ".create_agent_request_background_noise",
    "CreateAgentRequestConfigurationEndpoint": ".create_agent_request_configuration_endpoint",
    "CreateAgentRequestTemplateVariablesValue": ".create_agent_request_template_variables_value",
    "CreateAgentRequestToolsItem": ".create_agent_request_tools_item",
    "DtmfPayload": ".dtmf_payload",
    "Error": ".error",
    "ErrorPayload": ".error_payload",
    "ErrorPayloadError": ".error_payload_error",
    "ExtractionField": ".extraction_field",
    "ExtractionFieldType": ".extraction_field_type",
    "ExtractionSchema": ".extraction_schema",
    "InputCancelledPayload": ".input_cancelled_payload",
    "InputTextPayload": ".input_text_payload",
    "IsUserSpeakingPayload": ".is_user_speaking_payload",
    "LanguageCode": ".language_code",
    "OutboundCallConfig": ".outbound_call_config",
    "OutboundCallConfigToolsItem": ".outbound_call_config_tools_item",
    "Project": ".project",
    "ProjectDefaultAgent": ".project_default_agent",
    "ReadyToStartConversationPayload": ".ready_to_start_conversation_payload",
    "SetExternalIdPayload": ".set_external_id_payload",
    "SetTwilioCallSidPayload": ".set_twilio_call_sid_payload",
    "Task": ".task",
    "Tool": ".tool",
    "ToolCallInterruptedPayload": ".tool_call_interrupted_payload",
    "ToolCallOutputPayload": ".tool_call_output_payload",
    "ToolCallOutputProcessedPayload": ".tool_call_output_processed_payload",
    "ToolCallOutputProcessedPayloadTool": ".tool_call_output_processed_payload_tool",
    "ToolCallPayload": ".tool_call_payload",
    "ToolEndpointMethod": ".tool_endpoint_method",
    "ToolExecutionMode": ".tool_execution_mode",
    "ToolParameter": ".tool_parameter",
    "ToolParameterItemType": ".tool_parameter_item_type",
    "ToolParameterLocation": ".tool_parameter_location",
    "ToolParameterType": ".tool_parameter_type",
    "ToolProject": ".tool_project",
    "ToolType": ".tool_type",
    "UpdateSystemPromptPayload": ".update_system_prompt_payload",
    "UserFinishedSpeakingPayload": ".user_finished_speaking_payload",
    "UserStartedSpeakingPayload": ".user_started_speaking_payload",
    "ValidationError": ".validation_error",
    "ValidationErrorError": ".validation_error_error",
    "Voice": ".voice",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "Agent",
    "AgentAudioFormat",
    "AgentBackgroundNoise",
    "AgentConfigurationEndpoint",
    "AgentProject",
    "AgentTemplateVariablesValue",
    "AgentToolsItem",
    "AssistantChoseNotToRespondPayload",
    "AssistantEndedConversationPayload",
    "AudioChunkPayload",
    "AudioChunkResponsePayload",
    "BadRequestErrorBody",
    "BasicError",
    "BasicErrorError",
    "ConfigPayload",
    "ConfigPayloadBackgroundNoise",
    "ConfigPayloadInputFormat",
    "ConfigPayloadOutputFormat",
    "Conversation",
    "ConversationAgent",
    "ConversationAnalysis",
    "ConversationBackgroundNoise",
    "ConversationCreatedPayload",
    "ConversationEndedBy",
    "ConversationEval",
    "ConversationEvalPrompt",
    "ConversationEvaluation",
    "ConversationEvaluationResult",
    "ConversationEvaluationResultPrompt",
    "ConversationEvaluationResultResult",
    "ConversationExtraction",
    "ConversationExtractionSchema",
    "ConversationItem",
    "ConversationItemRole",
    "ConversationItemToolCallsItem",
    "ConversationItemToolCallsItemTool",
    "ConversationProject",
    "CreateAgentRequest",
    "CreateAgentRequestAudioFormat",
    "CreateAgentRequestBackgroundNoise",
    "CreateAgentRequestConfigurationEndpoint",
    "CreateAgentRequestTemplateVariablesValue",
    "CreateAgentRequestToolsItem",
    "DtmfPayload",
    "Error",
    "ErrorPayload",
    "ErrorPayloadError",
    "ExtractionField",
    "ExtractionFieldType",
    "ExtractionSchema",
    "InputCancelledPayload",
    "InputTextPayload",
    "IsUserSpeakingPayload",
    "LanguageCode",
    "OutboundCallConfig",
    "OutboundCallConfigToolsItem",
    "Project",
    "ProjectDefaultAgent",
    "ReadyToStartConversationPayload",
    "SetExternalIdPayload",
    "SetTwilioCallSidPayload",
    "Task",
    "Tool",
    "ToolCallInterruptedPayload",
    "ToolCallOutputPayload",
    "ToolCallOutputProcessedPayload",
    "ToolCallOutputProcessedPayloadTool",
    "ToolCallPayload",
    "ToolEndpointMethod",
    "ToolExecutionMode",
    "ToolParameter",
    "ToolParameterItemType",
    "ToolParameterLocation",
    "ToolParameterType",
    "ToolProject",
    "ToolType",
    "UpdateSystemPromptPayload",
    "UserFinishedSpeakingPayload",
    "UserStartedSpeakingPayload",
    "ValidationError",
    "ValidationErrorError",
    "Voice",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ErrorPayloadErrorParams(typing_extensions.TypedDict):
    message: str
    """
    Error message
    """

    code: typing_extensions.NotRequired[str]
    """
    Error code
    """



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel


class AudioChunkPayload(UniversalBaseModel):
    type: typing.Literal["audio_chunk"] = "audio_chunk"
    audio: str = pydantic.Field()
    """
    Base64-encoded audio data (Int16Array for PCM, Uint8Array for mulaw)
    """

    iso_date_time: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    ISO 8601 timestamp (required for first chunk only)
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ..types.extraction_field_type import ExtractionFieldType


class ExtractionFieldParams(typing_extensions.TypedDict):
    name: str
    """
    The field name.
    """

    type: ExtractionFieldType
    """
    The field type.
    """

    description: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Description of the field.
    """



# This file was auto-generated by Fern from our API Definition.

import typing
from contextlib import asynccontextmanager, contextmanager
from json.decoder import JSONDecodeError

import httpx
import websockets.exceptions
import websockets.sync.client as websockets_sync_client
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.http_response import AsyncHttpResponse, HttpResponse
from ..core.jsonable_encoder import jsonable_encoder
from ..core.pydantic_utilities import parse_obj_as
from ..core.request_options import RequestOptions
from ..core.serialization import convert_and_respect_annotation_metadata
from ..errors.bad_request_error import BadRequestError
from ..errors.conflict_error import ConflictError
from ..errors.forbidden_error import ForbiddenError
from ..errors.gateway_timeout_error import GatewayTimeoutError
from ..errors.internal_server_error import InternalServerError
from ..errors.not_found_error import NotFoundError
from ..errors.unauthorized_error import UnauthorizedError
from ..requests.outbound_call_config import OutboundCallConfigParams
from ..types.basic_error import BasicError
from ..types.conversation_evaluation_result import ConversationEvaluationResult
from ..types.error import Error
from .socket_client import AsyncConversationsSocketClient, ConversationsSocketClient
from .types.conversations_cancel_response import ConversationsCancelResponse
from .types.conversations_extract_data_response import ConversationsExtractDataResponse
from .types.conversations_get_analysis_response import ConversationsGetAnalysisResponse
from .types.conversations_get_response import ConversationsGetResponse
from .types.conversations_list_evaluations_response import ConversationsListEvaluationsResponse
from .types.conversations_list_extractions_response import ConversationsListExtractionsResponse
from .types.conversations_list_response import ConversationsListResponse
from .types.conversations_outbound_call_response import ConversationsOutboundCallResponse
from .types.conversations_sip_outbound_call_response import ConversationsSipOutboundCallResponse

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class RawConversationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._client_wrapper = client_wrapper

    def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationsListResponse]:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsListResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
                "external_id": external_id,
                "duration_min": duration_min,
                "duration_max": duration_max,
                "started_at_min": started_at_min,
                "started_at_max": started_at_max,
                "before": before,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListResponse,
                    parse_obj_as(
                        type_=ConversationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsGetResponse]:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsGetResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsGetResponse,
                    parse_obj_as(
                        type_=ConversationsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsCancelResponse]:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsCancelResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/cancel",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsCancelResponse,
                    parse_obj_as(
                        type_=ConversationsCancelResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsGetAnalysisResponse]:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsGetAnalysisResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/analysis",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsGetAnalysisResponse,
                    parse_obj_as(
                        type_=ConversationsGetAnalysisResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsListExtractionsResponse]:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsListExtractionsResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/extractions",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListExtractionsResponse,
                    parse_obj_as(
                        type_=ConversationsListExtractionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsExtractDataResponse]:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsExtractDataResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/extractions",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "schema_id": schema_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsExtractDataResponse,
                    parse_obj_as(
                        type_=ConversationsExtractDataResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationsListEvaluationsResponse]:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsListEvaluationsResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/evals",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListEvaluationsResponse,
                    parse_obj_as(
                        type_=ConversationsListEvaluationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> HttpResponse[ConversationEvaluationResult]:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationEvaluationResult]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/evals",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "prompt_id": prompt_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationEvaluationResult,
                    parse_obj_as(
                        type_=ConversationEvaluationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationsOutboundCallResponse]:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsOutboundCallResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations/outbound_call",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "to_phone_number": to_phone_number,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=OutboundCallConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsOutboundCallResponse,
                    parse_obj_as(
                        type_=ConversationsOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    def sip_outbound_call(
        self,
        *,
        sip_address: str,
        from_phone_number: str,
        to_phone_number: str,
        token: typing.Optional[str] = None,
        downstream_websocket_url: typing.Optional[str] = None,
        sip_auth_username: typing.Optional[str] = None,
        sip_auth_password: typing.Optional[str] = None,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> HttpResponse[ConversationsSipOutboundCallResponse]:
        """
        Initiates a SIP outbound call using user-supplied SIP credentials in headers.

        Parameters
        ----------
        sip_address : str
            SIP address of the user's SIP trunk. Required.

        from_phone_number : str
            Caller ID phone number in E.164 format.

        to_phone_number : str
            Destination phone number in E.164 format.

        token : typing.Optional[str]
            Optional session token used for STS authorization.

        downstream_websocket_url : typing.Optional[str]
            Optional downstream STS WebSocket URL to override the default.

        sip_auth_username : typing.Optional[str]
            SIP auth username, if your provider requires it.

        sip_auth_password : typing.Optional[str]
            SIP auth password, if your provider requires it.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        HttpResponse[ConversationsSipOutboundCallResponse]
            Success response
        """
        _response = self._client_wrapper.httpx_client.request(
            "conversations/sip/outbound_call",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "token": token,
                "downstream_websocket_url": downstream_websocket_url,
            },
            json={
                "from_phone_number": from_phone_number,
                "to_phone_number": to_phone_number,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=OutboundCallConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "X-Sip-Address": str(sip_address) if sip_address is not None else None,
                "X-Sip-Auth-Username": str(sip_auth_username) if sip_auth_username is not None else None,
                "X-Sip-Auth-Password": str(sip_auth_password) if sip_auth_password is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsSipOutboundCallResponse,
                    parse_obj_as(
                        type_=ConversationsSipOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return HttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @contextmanager
    def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ConversationsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsSocketClient
        """
        ws_url = self._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield ConversationsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncRawConversationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._client_wrapper = client_wrapper

    async def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationsListResponse]:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsListResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            params={
                "project": project,
                "external_id": external_id,
                "duration_min": duration_min,
                "duration_max": duration_max,
                "started_at_min": started_at_min,
                "started_at_max": started_at_max,
                "before": before,
                "after": after,
                "limit": limit,
            },
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListResponse,
                    parse_obj_as(
                        type_=ConversationsListResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsGetResponse]:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsGetResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsGetResponse,
                    parse_obj_as(
                        type_=ConversationsGetResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsCancelResponse]:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsCancelResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/cancel",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsCancelResponse,
                    parse_obj_as(
                        type_=ConversationsCancelResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 409:
                raise ConflictError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 504:
                raise GatewayTimeoutError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        Error,
                        parse_obj_as(
                            type_=Error,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsGetAnalysisResponse]:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsGetAnalysisResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/analysis",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsGetAnalysisResponse,
                    parse_obj_as(
                        type_=ConversationsGetAnalysisResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsListExtractionsResponse]:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsListExtractionsResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/extractions",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListExtractionsResponse,
                    parse_obj_as(
                        type_=ConversationsListExtractionsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsExtractDataResponse]:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsExtractDataResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/extractions",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "schema_id": schema_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsExtractDataResponse,
                    parse_obj_as(
                        type_=ConversationsExtractDataResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationsListEvaluationsResponse]:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsListEvaluationsResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/evals",
            base_url=self._client_wrapper.get_environment().base,
            method="GET",
            request_options=request_options,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsListEvaluationsResponse,
                    parse_obj_as(
                        type_=ConversationsListEvaluationsResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> AsyncHttpResponse[ConversationEvaluationResult]:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationEvaluationResult]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            f"conversations/{jsonable_encoder(id)}/evals",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "prompt_id": prompt_id,
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationEvaluationResult,
                    parse_obj_as(
                        type_=ConversationEvaluationResult,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 403:
                raise ForbiddenError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationsOutboundCallResponse]:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsOutboundCallResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations/outbound_call",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            json={
                "to_phone_number": to_phone_number,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=OutboundCallConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsOutboundCallResponse,
                    parse_obj_as(
                        type_=ConversationsOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 404:
                raise NotFoundError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    async def sip_outbound_call(
        self,
        *,
        sip_address: str,
        from_phone_number: str,
        to_phone_number: str,
        token: typing.Optional[str] = None,
        downstream_websocket_url: typing.Optional[str] = None,
        sip_auth_username: typing.Optional[str] = None,
        sip_auth_password: typing.Optional[str] = None,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AsyncHttpResponse[ConversationsSipOutboundCallResponse]:
        """
        Initiates a SIP outbound call using user-supplied SIP credentials in headers.

        Parameters
        ----------
        sip_address : str
            SIP address of the user's SIP trunk. Required.

        from_phone_number : str
            Caller ID phone number in E.164 format.

        to_phone_number : str
            Destination phone number in E.164 format.

        token : typing.Optional[str]
            Optional session token used for STS authorization.

        downstream_websocket_url : typing.Optional[str]
            Optional downstream STS WebSocket URL to override the default.

        sip_auth_username : typing.Optional[str]
            SIP auth username, if your provider requires it.

        sip_auth_password : typing.Optional[str]
            SIP auth password, if your provider requires it.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncHttpResponse[ConversationsSipOutboundCallResponse]
            Success response
        """
        _response = await self._client_wrapper.httpx_client.request(
            "conversations/sip/outbound_call",
            base_url=self._client_wrapper.get_environment().base,
            method="POST",
            params={
                "token": token,
                "downstream_websocket_url": downstream_websocket_url,
            },
            json={
                "from_phone_number": from_phone_number,
                "to_phone_number": to_phone_number,
                "config": convert_and_respect_annotation_metadata(
                    object_=config, annotation=OutboundCallConfigParams, direction="write"
                ),
            },
            headers={
                "content-type": "application/json",
                "X-Sip-Address": str(sip_address) if sip_address is not None else None,
                "X-Sip-Auth-Username": str(sip_auth_username) if sip_auth_username is not None else None,
                "X-Sip-Auth-Password": str(sip_auth_password) if sip_auth_password is not None else None,
            },
            request_options=request_options,
            omit=OMIT,
        )
        try:
            if 200 <= _response.status_code < 300:
                _data = typing.cast(
                    ConversationsSipOutboundCallResponse,
                    parse_obj_as(
                        type_=ConversationsSipOutboundCallResponse,  # type: ignore
                        object_=_response.json(),
                    ),
                )
                return AsyncHttpResponse(response=_response, data=_data)
            if _response.status_code == 400:
                raise BadRequestError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        typing.Optional[typing.Any],
                        parse_obj_as(
                            type_=typing.Optional[typing.Any],  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 401:
                raise UnauthorizedError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            if _response.status_code == 500:
                raise InternalServerError(
                    headers=dict(_response.headers),
                    body=typing.cast(
                        BasicError,
                        parse_obj_as(
                            type_=BasicError,  # type: ignore
                            object_=_response.json(),
                        ),
                    ),
                )
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response.text)
        raise ApiError(status_code=_response.status_code, headers=dict(_response.headers), body=_response_json)

    @asynccontextmanager
    async def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncConversationsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncConversationsSocketClient
        """
        ws_url = self._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncConversationsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.tool import ToolParams


class ToolsGetResponseParams(typing_extensions.TypedDict):
    tool: ToolParams



# This file was auto-generated by Fern from our API Definition.

import typing

from .conversations_list_response_conversation import ConversationsListResponseConversationParams
from .conversations_list_response_conversations import ConversationsListResponseConversationsParams

ConversationsListResponseParams = typing.Union[
    ConversationsListResponseConversationsParams, ConversationsListResponseConversationParams
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from .basic_error_error import BasicErrorErrorParams


class BasicErrorParams(typing_extensions.TypedDict):
    error: typing_extensions.NotRequired[BasicErrorErrorParams]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.voice import VoiceParams


class VoicesGetResponseParams(typing_extensions.TypedDict):
    voice: VoiceParams



# This file was auto-generated by Fern from our API Definition.

import typing
from contextlib import asynccontextmanager, contextmanager

import httpx
import websockets.exceptions
import websockets.sync.client as websockets_sync_client
from ..core.api_error import ApiError
from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..requests.outbound_call_config import OutboundCallConfigParams
from ..types.conversation_evaluation_result import ConversationEvaluationResult
from .raw_client import AsyncRawConversationsClient, RawConversationsClient
from .socket_client import AsyncConversationsSocketClient, ConversationsSocketClient
from .types.conversations_cancel_response import ConversationsCancelResponse
from .types.conversations_extract_data_response import ConversationsExtractDataResponse
from .types.conversations_get_analysis_response import ConversationsGetAnalysisResponse
from .types.conversations_get_response import ConversationsGetResponse
from .types.conversations_list_evaluations_response import ConversationsListEvaluationsResponse
from .types.conversations_list_extractions_response import ConversationsListExtractionsResponse
from .types.conversations_list_response import ConversationsListResponse
from .types.conversations_outbound_call_response import ConversationsOutboundCallResponse
from .types.conversations_sip_outbound_call_response import ConversationsSipOutboundCallResponse

try:
    from websockets.legacy.client import connect as websockets_client_connect  # type: ignore
except ImportError:
    from websockets import connect as websockets_client_connect  # type: ignore

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ConversationsClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawConversationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawConversationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawConversationsClient
        """
        return self._raw_client

    def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsListResponse:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.list()
        """
        _response = self._raw_client.list(
            project=project,
            external_id=external_id,
            duration_min=duration_min,
            duration_max=duration_max,
            started_at_min=started_at_min,
            started_at_max=started_at_max,
            before=before,
            after=after,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    def get(self, id: str, *, request_options: typing.Optional[RequestOptions] = None) -> ConversationsGetResponse:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.get(
            id="id",
        )
        """
        _response = self._raw_client.get(id, request_options=request_options)
        return _response.data

    def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsCancelResponse:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsCancelResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.cancel(
            id="id",
        )
        """
        _response = self._raw_client.cancel(id, request_options=request_options)
        return _response.data

    def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetAnalysisResponse:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetAnalysisResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.get_analysis(
            id="id",
        )
        """
        _response = self._raw_client.get_analysis(id, request_options=request_options)
        return _response.data

    def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListExtractionsResponse:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListExtractionsResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.list_extractions(
            id="id",
        )
        """
        _response = self._raw_client.list_extractions(id, request_options=request_options)
        return _response.data

    def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsExtractDataResponse:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsExtractDataResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.extract_data(
            id="id",
            schema_id="conv_extract_schema_6458e4ac-533c-4bdf-8e6d-c2f06f87fd5c",
        )
        """
        _response = self._raw_client.extract_data(id, schema_id=schema_id, request_options=request_options)
        return _response.data

    def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListEvaluationsResponse:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListEvaluationsResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.list_evaluations(
            id="id",
        )
        """
        _response = self._raw_client.list_evaluations(id, request_options=request_options)
        return _response.data

    def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationEvaluationResult:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationEvaluationResult
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.evaluate(
            id="id",
            prompt_id="conv_eval_prompt_d7cfe45d-35db-4ef6-a254-81ab1da76ce0",
        )
        """
        _response = self._raw_client.evaluate(id, prompt_id=prompt_id, request_options=request_options)
        return _response.data

    def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsOutboundCallResponse:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsOutboundCallResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.outbound_call(
            to_phone_number="+19189397081",
            config={
                "agent": "support-agent",
                "welcome_message": "Hi {{customer_name}}. How can I help you today?",
                "system_prompt": "You are an expert in {{subject}}. Be friendly, helpful and concise.",
                "template_variables": {"customer_name": "David", "subject": "Chess"},
                "voice_id": "grant",
                "no_input_poke_sec": 30,
                "no_input_poke_text": "Are you still there?",
                "no_input_end_conversation_sec": 180,
                "languages": ["en", "es"],
                "boosted_keywords": ["Load ID", "dispatch"],
                "tools": [],
            },
        )
        """
        _response = self._raw_client.outbound_call(
            to_phone_number=to_phone_number, config=config, request_options=request_options
        )
        return _response.data

    def sip_outbound_call(
        self,
        *,
        sip_address: str,
        from_phone_number: str,
        to_phone_number: str,
        token: typing.Optional[str] = None,
        downstream_websocket_url: typing.Optional[str] = None,
        sip_auth_username: typing.Optional[str] = None,
        sip_auth_password: typing.Optional[str] = None,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsSipOutboundCallResponse:
        """
        Initiates a SIP outbound call using user-supplied SIP credentials in headers.

        Parameters
        ----------
        sip_address : str
            SIP address of the user's SIP trunk. Required.

        from_phone_number : str
            Caller ID phone number in E.164 format.

        to_phone_number : str
            Destination phone number in E.164 format.

        token : typing.Optional[str]
            Optional session token used for STS authorization.

        downstream_websocket_url : typing.Optional[str]
            Optional downstream STS WebSocket URL to override the default.

        sip_auth_username : typing.Optional[str]
            SIP auth username, if your provider requires it.

        sip_auth_password : typing.Optional[str]
            SIP auth password, if your provider requires it.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsSipOutboundCallResponse
            Success response

        Examples
        --------
        from phonic import Phonic

        client = Phonic(
            api_key="YOUR_API_KEY",
        )
        client.conversations.sip_outbound_call(
            sip_address="X-Sip-Address",
            from_phone_number="from_phone_number",
            to_phone_number="to_phone_number",
        )
        """
        _response = self._raw_client.sip_outbound_call(
            sip_address=sip_address,
            from_phone_number=from_phone_number,
            to_phone_number=to_phone_number,
            token=token,
            downstream_websocket_url=downstream_websocket_url,
            sip_auth_username=sip_auth_username,
            sip_auth_password=sip_auth_password,
            config=config,
            request_options=request_options,
        )
        return _response.data

    @contextmanager
    def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.Iterator[ConversationsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            with websockets_sync_client.connect(ws_url, additional_headers=headers) as protocol:
                yield ConversationsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )


class AsyncConversationsClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawConversationsClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawConversationsClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawConversationsClient
        """
        return self._raw_client

    async def list(
        self,
        *,
        project: typing.Optional[str] = None,
        external_id: typing.Optional[str] = None,
        duration_min: typing.Optional[int] = None,
        duration_max: typing.Optional[int] = None,
        started_at_min: typing.Optional[str] = None,
        started_at_max: typing.Optional[str] = None,
        before: typing.Optional[str] = None,
        after: typing.Optional[str] = None,
        limit: typing.Optional[int] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsListResponse:
        """
        Returns conversations with optional filtering.

        Parameters
        ----------
        project : typing.Optional[str]
            The name of the project to list conversations for.

        external_id : typing.Optional[str]
            Filter by external ID to get a specific conversation.

        duration_min : typing.Optional[int]
            Minimum duration in seconds.

        duration_max : typing.Optional[int]
            Maximum duration in seconds.

        started_at_min : typing.Optional[str]
            Minimum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        started_at_max : typing.Optional[str]
            Maximum start date/time. Valid examples: `2025-04-17`, `2025-04-17T02:48:52.708Z`

        before : typing.Optional[str]
            Cursor for pagination (before).

        after : typing.Optional[str]
            Cursor for pagination (after).

        limit : typing.Optional[int]
            Maximum number of conversations to return.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.list()


        asyncio.run(main())
        """
        _response = await self._raw_client.list(
            project=project,
            external_id=external_id,
            duration_min=duration_min,
            duration_max=duration_max,
            started_at_min=started_at_min,
            started_at_max=started_at_max,
            before=before,
            after=after,
            limit=limit,
            request_options=request_options,
        )
        return _response.data

    async def get(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetResponse:
        """
        Returns a conversation by ID.

        Parameters
        ----------
        id : str
            The ID of the conversation to get.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.get(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get(id, request_options=request_options)
        return _response.data

    async def cancel(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsCancelResponse:
        """
        Cancels an active conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to cancel.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsCancelResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.cancel(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel(id, request_options=request_options)
        return _response.data

    async def get_analysis(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsGetAnalysisResponse:
        """
        Returns an analysis of the specified conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to analyze.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsGetAnalysisResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.get_analysis(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_analysis(id, request_options=request_options)
        return _response.data

    async def list_extractions(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListExtractionsResponse:
        """
        Returns all extractions for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get extractions for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListExtractionsResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.list_extractions(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_extractions(id, request_options=request_options)
        return _response.data

    async def extract_data(
        self, id: str, *, schema_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsExtractDataResponse:
        """
        Extracts data from a conversation using a schema.

        Parameters
        ----------
        id : str
            The ID of the conversation to extract data from.

        schema_id : str
            ID of the extraction schema to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsExtractDataResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.extract_data(
                id="id",
                schema_id="conv_extract_schema_6458e4ac-533c-4bdf-8e6d-c2f06f87fd5c",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.extract_data(id, schema_id=schema_id, request_options=request_options)
        return _response.data

    async def list_evaluations(
        self, id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationsListEvaluationsResponse:
        """
        Returns all evaluations for a conversation.

        Parameters
        ----------
        id : str
            The ID of the conversation to get evaluations for.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsListEvaluationsResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.list_evaluations(
                id="id",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_evaluations(id, request_options=request_options)
        return _response.data

    async def evaluate(
        self, id: str, *, prompt_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> ConversationEvaluationResult:
        """
        Evaluates a conversation using an evaluation prompt.

        Parameters
        ----------
        id : str
            The ID of the conversation to evaluate.

        prompt_id : str
            ID of the evaluation prompt to use.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationEvaluationResult
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.evaluate(
                id="id",
                prompt_id="conv_eval_prompt_d7cfe45d-35db-4ef6-a254-81ab1da76ce0",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.evaluate(id, prompt_id=prompt_id, request_options=request_options)
        return _response.data

    async def outbound_call(
        self,
        *,
        to_phone_number: str,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsOutboundCallResponse:
        """
        Initiates a call to a given phone number using Phonic's Twilio account.

        Parameters
        ----------
        to_phone_number : str
            The phone number to call in E.164 format.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsOutboundCallResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.outbound_call(
                to_phone_number="+19189397081",
                config={
                    "agent": "support-agent",
                    "welcome_message": "Hi {{customer_name}}. How can I help you today?",
                    "system_prompt": "You are an expert in {{subject}}. Be friendly, helpful and concise.",
                    "template_variables": {
                        "customer_name": "David",
                        "subject": "Chess",
                    },
                    "voice_id": "grant",
                    "no_input_poke_sec": 30,
                    "no_input_poke_text": "Are you still there?",
                    "no_input_end_conversation_sec": 180,
                    "languages": ["en", "es"],
                    "boosted_keywords": ["Load ID", "dispatch"],
                    "tools": [],
                },
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.outbound_call(
            to_phone_number=to_phone_number, config=config, request_options=request_options
        )
        return _response.data

    async def sip_outbound_call(
        self,
        *,
        sip_address: str,
        from_phone_number: str,
        to_phone_number: str,
        token: typing.Optional[str] = None,
        downstream_websocket_url: typing.Optional[str] = None,
        sip_auth_username: typing.Optional[str] = None,
        sip_auth_password: typing.Optional[str] = None,
        config: typing.Optional[OutboundCallConfigParams] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> ConversationsSipOutboundCallResponse:
        """
        Initiates a SIP outbound call using user-supplied SIP credentials in headers.

        Parameters
        ----------
        sip_address : str
            SIP address of the user's SIP trunk. Required.

        from_phone_number : str
            Caller ID phone number in E.164 format.

        to_phone_number : str
            Destination phone number in E.164 format.

        token : typing.Optional[str]
            Optional session token used for STS authorization.

        downstream_websocket_url : typing.Optional[str]
            Optional downstream STS WebSocket URL to override the default.

        sip_auth_username : typing.Optional[str]
            SIP auth username, if your provider requires it.

        sip_auth_password : typing.Optional[str]
            SIP auth password, if your provider requires it.

        config : typing.Optional[OutboundCallConfigParams]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ConversationsSipOutboundCallResponse
            Success response

        Examples
        --------
        import asyncio

        from phonic import AsyncPhonic

        client = AsyncPhonic(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.conversations.sip_outbound_call(
                sip_address="X-Sip-Address",
                from_phone_number="from_phone_number",
                to_phone_number="to_phone_number",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.sip_outbound_call(
            sip_address=sip_address,
            from_phone_number=from_phone_number,
            to_phone_number=to_phone_number,
            token=token,
            downstream_websocket_url=downstream_websocket_url,
            sip_auth_username=sip_auth_username,
            sip_auth_password=sip_auth_password,
            config=config,
            request_options=request_options,
        )
        return _response.data

    @asynccontextmanager
    async def connect(
        self,
        *,
        downstream_websocket_url: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> typing.AsyncIterator[AsyncConversationsSocketClient]:
        """
        Main STS WebSocket channel for real-time voice conversations

        Parameters
        ----------
        downstream_websocket_url : typing.Optional[str]
            Custom downstream WebSocket URL

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AsyncConversationsSocketClient
        """
        ws_url = self._raw_client._client_wrapper.get_environment().production + "/v1/sts/ws"
        query_params = httpx.QueryParams()
        if downstream_websocket_url is not None:
            query_params = query_params.add("downstream_websocket_url", downstream_websocket_url)
        ws_url = ws_url + f"?{query_params}"
        headers = self._raw_client._client_wrapper.get_headers()
        if request_options and "additional_headers" in request_options:
            headers.update(request_options["additional_headers"])
        try:
            async with websockets_client_connect(ws_url, extra_headers=headers) as protocol:
                yield AsyncConversationsSocketClient(websocket=protocol)
        except websockets.exceptions.InvalidStatusCode as exc:
            status_code: int = exc.status_code
            if status_code == 401:
                raise ApiError(
                    status_code=status_code,
                    headers=dict(headers),
                    body="Websocket initialized with invalid credentials.",
                )
            raise ApiError(
                status_code=status_code,
                headers=dict(headers),
                body="Unexpected error when initializing websocket connection.",
            )



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations


class PhonicEnvironment:
    DEFAULT: PhonicEnvironment

    def __init__(self, *, base: str, production: str):
        self.base = base
        self.production = production


PhonicEnvironment.DEFAULT = PhonicEnvironment(base="https://api.phonic.co/v1", production="wss://api.phonic.co")



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .conversation_item_tool_calls_item_tool import ConversationItemToolCallsItemTool


class ConversationItemToolCallsItem(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The tool call ID.
    """

    tool: ConversationItemToolCallsItemTool
    endpoint_method: typing.Optional[str] = pydantic.Field(default=None)
    """
    HTTP method for webhook tool calls.
    """

    endpoint_url: typing.Optional[str] = pydantic.Field(default=None)
    """
    URL for webhook tool calls.
    """

    endpoint_headers: typing.Optional[typing.Dict[str, typing.Optional[str]]] = pydantic.Field(default=None)
    """
    Headers for webhook tool calls.
    """

    endpoint_timeout_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Timeout in milliseconds for webhook tool calls.
    """

    endpoint_called_at: typing.Optional[dt.datetime] = pydantic.Field(default=None)
    """
    When the webhook endpoint was called (null on error).
    """

    query_params: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    Query parameters for webhook tool calls (null on error or when no params).
    """

    response_status_code: typing.Optional[float] = pydantic.Field(default=None)
    """
    HTTP response status code for webhook tool calls (null on error).
    """

    tool_call_output_timeout_ms: typing.Optional[float] = pydantic.Field(default=None)
    """
    Timeout in milliseconds for websocket tool calls.
    """

    request_body: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    The request body sent to the tool.
    """

    response_body: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]] = pydantic.Field(default=None)
    """
    The response body received from the tool.
    """

    timed_out: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Whether the tool call timed out.
    """

    error_message: typing.Optional[str] = pydantic.Field(default=None)
    """
    Error message if the tool call failed.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from .extraction_field import ExtractionFieldParams


class ExtractionSchemaParams(typing_extensions.TypedDict):
    id: str
    """
    The ID of the extraction schema.
    """

    name: str
    """
    The name of the extraction schema.
    """

    prompt: str
    """
    Instructions for how to extract data from conversations.
    """

    fields: typing.Sequence[ExtractionFieldParams]
    """
    Array of field definitions specifying what data to extract.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions


class SetTwilioCallSidPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["set_twilio_call_sid"]
    encrypted_data: str
    """
    Encrypted Twilio call data
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .basic_error import BasicError
from .validation_error import ValidationError

BadRequestErrorBody = typing.Union[ValidationError, BasicError]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from .extraction_field import ExtractionField


class ExtractionSchema(UniversalBaseModel):
    id: str = pydantic.Field()
    """
    The ID of the extraction schema.
    """

    name: str = pydantic.Field()
    """
    The name of the extraction schema.
    """

    prompt: str = pydantic.Field()
    """
    Instructions for how to extract data from conversations.
    """

    fields: typing.List[ExtractionField] = pydantic.Field()
    """
    Array of field definitions specifying what data to extract.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .agent import AgentParams
    from .agent_configuration_endpoint import AgentConfigurationEndpointParams
    from .agent_project import AgentProjectParams
    from .agent_template_variables_value import AgentTemplateVariablesValueParams
    from .agent_tools_item import AgentToolsItemParams
    from .assistant_chose_not_to_respond_payload import AssistantChoseNotToRespondPayloadParams
    from .assistant_ended_conversation_payload import AssistantEndedConversationPayloadParams
    from .audio_chunk_payload import AudioChunkPayloadParams
    from .audio_chunk_response_payload import AudioChunkResponsePayloadParams
    from .bad_request_error_body import BadRequestErrorBodyParams
    from .basic_error import BasicErrorParams
    from .basic_error_error import BasicErrorErrorParams
    from .config_payload import ConfigPayloadParams
    from .conversation import ConversationParams
    from .conversation_agent import ConversationAgentParams
    from .conversation_analysis import ConversationAnalysisParams
    from .conversation_created_payload import ConversationCreatedPayloadParams
    from .conversation_eval_prompt import ConversationEvalPromptParams
    from .conversation_evaluation import ConversationEvaluationParams
    from .conversation_evaluation_result import ConversationEvaluationResultParams
    from .conversation_evaluation_result_prompt import ConversationEvaluationResultPromptParams
    from .conversation_extraction import ConversationExtractionParams
    from .conversation_extraction_schema import ConversationExtractionSchemaParams
    from .conversation_item import ConversationItemParams
    from .conversation_item_tool_calls_item import ConversationItemToolCallsItemParams
    from .conversation_item_tool_calls_item_tool import ConversationItemToolCallsItemToolParams
    from .conversation_project import ConversationProjectParams
    from .create_agent_request import CreateAgentRequestParams
    from .create_agent_request_configuration_endpoint import CreateAgentRequestConfigurationEndpointParams
    from .create_agent_request_template_variables_value import CreateAgentRequestTemplateVariablesValueParams
    from .create_agent_request_tools_item import CreateAgentRequestToolsItemParams
    from .dtmf_payload import DtmfPayloadParams
    from .error import ErrorParams
    from .error_payload import ErrorPayloadParams
    from .error_payload_error import ErrorPayloadErrorParams
    from .extraction_field import ExtractionFieldParams
    from .extraction_schema import ExtractionSchemaParams
    from .input_cancelled_payload import InputCancelledPayloadParams
    from .input_text_payload import InputTextPayloadParams
    from .is_user_speaking_payload import IsUserSpeakingPayloadParams
    from .outbound_call_config import OutboundCallConfigParams
    from .outbound_call_config_tools_item import OutboundCallConfigToolsItemParams
    from .project import ProjectParams
    from .project_default_agent import ProjectDefaultAgentParams
    from .ready_to_start_conversation_payload import ReadyToStartConversationPayloadParams
    from .set_external_id_payload import SetExternalIdPayloadParams
    from .set_twilio_call_sid_payload import SetTwilioCallSidPayloadParams
    from .task import TaskParams
    from .tool import ToolParams
    from .tool_call_interrupted_payload import ToolCallInterruptedPayloadParams
    from .tool_call_output_payload import ToolCallOutputPayloadParams
    from .tool_call_output_processed_payload import ToolCallOutputProcessedPayloadParams
    from .tool_call_output_processed_payload_tool import ToolCallOutputProcessedPayloadToolParams
    from .tool_call_payload import ToolCallPayloadParams
    from .tool_parameter import ToolParameterParams
    from .tool_project import ToolProjectParams
    from .update_system_prompt_payload import UpdateSystemPromptPayloadParams
    from .user_finished_speaking_payload import UserFinishedSpeakingPayloadParams
    from .user_started_speaking_payload import UserStartedSpeakingPayloadParams
    from .validation_error import ValidationErrorParams
    from .validation_error_error import ValidationErrorErrorParams
    from .voice import VoiceParams
_dynamic_imports: typing.Dict[str, str] = {
    "AgentConfigurationEndpointParams": ".agent_configuration_endpoint",
    "AgentParams": ".agent",
    "AgentProjectParams": ".agent_project",
    "AgentTemplateVariablesValueParams": ".agent_template_variables_value",
    "AgentToolsItemParams": ".agent_tools_item",
    "AssistantChoseNotToRespondPayloadParams": ".assistant_chose_not_to_respond_payload",
    "AssistantEndedConversationPayloadParams": ".assistant_ended_conversation_payload",
    "AudioChunkPayloadParams": ".audio_chunk_payload",
    "AudioChunkResponsePayloadParams": ".audio_chunk_response_payload",
    "BadRequestErrorBodyParams": ".bad_request_error_body",
    "BasicErrorErrorParams": ".basic_error_error",
    "BasicErrorParams": ".basic_error",
    "ConfigPayloadParams": ".config_payload",
    "ConversationAgentParams": ".conversation_agent",
    "ConversationAnalysisParams": ".conversation_analysis",
    "ConversationCreatedPayloadParams": ".conversation_created_payload",
    "ConversationEvalPromptParams": ".conversation_eval_prompt",
    "ConversationEvaluationParams": ".conversation_evaluation",
    "ConversationEvaluationResultParams": ".conversation_evaluation_result",
    "ConversationEvaluationResultPromptParams": ".conversation_evaluation_result_prompt",
    "ConversationExtractionParams": ".conversation_extraction",
    "ConversationExtractionSchemaParams": ".conversation_extraction_schema",
    "ConversationItemParams": ".conversation_item",
    "ConversationItemToolCallsItemParams": ".conversation_item_tool_calls_item",
    "ConversationItemToolCallsItemToolParams": ".conversation_item_tool_calls_item_tool",
    "ConversationParams": ".conversation",
    "ConversationProjectParams": ".conversation_project",
    "CreateAgentRequestConfigurationEndpointParams": ".create_agent_request_configuration_endpoint",
    "CreateAgentRequestParams": ".create_agent_request",
    "CreateAgentRequestTemplateVariablesValueParams": ".create_agent_request_template_variables_value",
    "CreateAgentRequestToolsItemParams": ".create_agent_request_tools_item",
    "DtmfPayloadParams": ".dtmf_payload",
    "ErrorParams": ".error",
    "ErrorPayloadErrorParams": ".error_payload_error",
    "ErrorPayloadParams": ".error_payload",
    "ExtractionFieldParams": ".extraction_field",
    "ExtractionSchemaParams": ".extraction_schema",
    "InputCancelledPayloadParams": ".input_cancelled_payload",
    "InputTextPayloadParams": ".input_text_payload",
    "IsUserSpeakingPayloadParams": ".is_user_speaking_payload",
    "OutboundCallConfigParams": ".outbound_call_config",
    "OutboundCallConfigToolsItemParams": ".outbound_call_config_tools_item",
    "ProjectDefaultAgentParams": ".project_default_agent",
    "ProjectParams": ".project",
    "ReadyToStartConversationPayloadParams": ".ready_to_start_conversation_payload",
    "SetExternalIdPayloadParams": ".set_external_id_payload",
    "SetTwilioCallSidPayloadParams": ".set_twilio_call_sid_payload",
    "TaskParams": ".task",
    "ToolCallInterruptedPayloadParams": ".tool_call_interrupted_payload",
    "ToolCallOutputPayloadParams": ".tool_call_output_payload",
    "ToolCallOutputProcessedPayloadParams": ".tool_call_output_processed_payload",
    "ToolCallOutputProcessedPayloadToolParams": ".tool_call_output_processed_payload_tool",
    "ToolCallPayloadParams": ".tool_call_payload",
    "ToolParameterParams": ".tool_parameter",
    "ToolParams": ".tool",
    "ToolProjectParams": ".tool_project",
    "UpdateSystemPromptPayloadParams": ".update_system_prompt_payload",
    "UserFinishedSpeakingPayloadParams": ".user_finished_speaking_payload",
    "UserStartedSpeakingPayloadParams": ".user_started_speaking_payload",
    "ValidationErrorErrorParams": ".validation_error_error",
    "ValidationErrorParams": ".validation_error",
    "VoiceParams": ".voice",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentConfigurationEndpointParams",
    "AgentParams",
    "AgentProjectParams",
    "AgentTemplateVariablesValueParams",
    "AgentToolsItemParams",
    "AssistantChoseNotToRespondPayloadParams",
    "AssistantEndedConversationPayloadParams",
    "AudioChunkPayloadParams",
    "AudioChunkResponsePayloadParams",
    "BadRequestErrorBodyParams",
    "BasicErrorErrorParams",
    "BasicErrorParams",
    "ConfigPayloadParams",
    "ConversationAgentParams",
    "ConversationAnalysisParams",
    "ConversationCreatedPayloadParams",
    "ConversationEvalPromptParams",
    "ConversationEvaluationParams",
    "ConversationEvaluationResultParams",
    "ConversationEvaluationResultPromptParams",
    "ConversationExtractionParams",
    "ConversationExtractionSchemaParams",
    "ConversationItemParams",
    "ConversationItemToolCallsItemParams",
    "ConversationItemToolCallsItemToolParams",
    "ConversationParams",
    "ConversationProjectParams",
    "CreateAgentRequestConfigurationEndpointParams",
    "CreateAgentRequestParams",
    "CreateAgentRequestTemplateVariablesValueParams",
    "CreateAgentRequestToolsItemParams",
    "DtmfPayloadParams",
    "ErrorParams",
    "ErrorPayloadErrorParams",
    "ErrorPayloadParams",
    "ExtractionFieldParams",
    "ExtractionSchemaParams",
    "InputCancelledPayloadParams",
    "InputTextPayloadParams",
    "IsUserSpeakingPayloadParams",
    "OutboundCallConfigParams",
    "OutboundCallConfigToolsItemParams",
    "ProjectDefaultAgentParams",
    "ProjectParams",
    "ReadyToStartConversationPayloadParams",
    "SetExternalIdPayloadParams",
    "SetTwilioCallSidPayloadParams",
    "TaskParams",
    "ToolCallInterruptedPayloadParams",
    "ToolCallOutputPayloadParams",
    "ToolCallOutputProcessedPayloadParams",
    "ToolCallOutputProcessedPayloadToolParams",
    "ToolCallPayloadParams",
    "ToolParameterParams",
    "ToolParams",
    "ToolProjectParams",
    "UpdateSystemPromptPayloadParams",
    "UserFinishedSpeakingPayloadParams",
    "UserStartedSpeakingPayloadParams",
    "ValidationErrorErrorParams",
    "ValidationErrorParams",
    "VoiceParams",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from .tool_call_output_processed_payload_tool import ToolCallOutputProcessedPayloadToolParams


class ToolCallOutputProcessedPayloadParams(typing_extensions.TypedDict):
    type: typing.Literal["tool_call_output_processed"]
    tool_call_id: str
    """
    ID of the completed tool call
    """

    tool: ToolCallOutputProcessedPayloadToolParams
    endpoint_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Webhook endpoint URL (null for WebSocket tools)
    """

    endpoint_timeout_ms: typing_extensions.NotRequired[typing.Optional[float]]
    """
    Webhook timeout in milliseconds (null for WebSocket tools)
    """

    endpoint_called_at: typing_extensions.NotRequired[typing.Optional[dt.datetime]]
    """
    When webhook was called (null for WebSocket tools)
    """

    request_body: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]]
    """
    Webhook request body (null for WebSocket tools)
    """

    response_body: typing_extensions.NotRequired[typing.Optional[typing.Any]]
    parameters: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]]
    """
    WebSocket tool parameters (null for webhook tools)
    """

    output: typing_extensions.NotRequired[typing.Optional[typing.Any]]
    response_status_code: typing_extensions.NotRequired[typing.Optional[float]]
    """
    Webhook HTTP status code (null for WebSocket tools)
    """

    timed_out: typing_extensions.NotRequired[typing.Optional[bool]]
    """
    Whether the tool call timed out
    """

    error_message: typing_extensions.NotRequired[typing.Optional[str]]
    """
    Error message if tool call failed
    """



# This file was auto-generated by Fern from our API Definition.

import typing

import typing_extensions
from ...requests.conversation import ConversationParams


class ConversationsListResponseConversationsParams(typing_extensions.TypedDict):
    conversations: typing.Sequence[ConversationParams]



# This file was auto-generated by Fern from our API Definition.

import typing

UpdateAgentRequestToolsItemParams = typing.Union[
    typing.Literal["keypad_input"], typing.Literal["natural_conversation_ending"], str
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.conversation_analysis import ConversationAnalysisParams


class ConversationsGetAnalysisResponseParams(typing_extensions.TypedDict):
    analysis: ConversationAnalysisParams



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import typing_extensions
from .conversation_item_tool_calls_item_tool import ConversationItemToolCallsItemToolParams


class ConversationItemToolCallsItemParams(typing_extensions.TypedDict):
    id: str
    """
    The tool call ID.
    """

    tool: ConversationItemToolCallsItemToolParams
    endpoint_method: typing_extensions.NotRequired[typing.Optional[str]]
    """
    HTTP method for webhook tool calls.
    """

    endpoint_url: typing_extensions.NotRequired[typing.Optional[str]]
    """
    URL for webhook tool calls.
    """

    endpoint_headers: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[str]]]]
    """
    Headers for webhook tool calls.
    """

    endpoint_timeout_ms: typing_extensions.NotRequired[typing.Optional[float]]
    """
    Timeout in milliseconds for webhook tool calls.
    """

    endpoint_called_at: typing_extensions.NotRequired[typing.Optional[dt.datetime]]
    """
    When the webhook endpoint was called (null on error).
    """

    query_params: typing_extensions.NotRequired[typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]]
    """
    Query parameters for webhook tool calls (null on error or when no params).
    """

    response_status_code: typing_extensions.NotRequired[typing.Optional[float]]
    """
    HTTP response status code for webhook tool calls (null on error).
    """

    tool_call_output_timeout_ms: typing_extensions.NotRequired[typing.Optional[float]]
    """
    Timeout in milliseconds for websocket tool calls.
    """

    request_body: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    The request body sent to the tool.
    """

    response_body: typing.Optional[typing.Dict[str, typing.Optional[typing.Any]]]
    """
    The response body received from the tool.
    """

    timed_out: typing.Optional[bool]
    """
    Whether the tool call timed out.
    """

    error_message: typing.Optional[str]
    """
    Error message if the tool call failed.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.agent import AgentParams


class AgentsUpsertResponseParams(typing_extensions.TypedDict):
    agent: AgentParams
    inserted: bool
    """
    Whether the agent was inserted.
    """

    updated: bool
    """
    Whether the agent was updated.
    """



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ConversationProjectParams(typing_extensions.TypedDict):
    """
    The project associated with the conversation.
    """

    id: str
    """
    The ID of the project.
    """

    name: str
    """
    The name of the project.
    """



# This file was auto-generated by Fern from our API Definition.

import typing

from .basic_error import BasicErrorParams
from .validation_error import ValidationErrorParams

BadRequestErrorBodyParams = typing.Union[ValidationErrorParams, BasicErrorParams]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .agents_create_response import AgentsCreateResponseParams
    from .agents_delete_response import AgentsDeleteResponseParams
    from .agents_get_response import AgentsGetResponseParams
    from .agents_list_response import AgentsListResponseParams
    from .agents_update_response import AgentsUpdateResponseParams
    from .agents_upsert_response import AgentsUpsertResponseParams
    from .update_agent_request_configuration_endpoint import UpdateAgentRequestConfigurationEndpointParams
    from .update_agent_request_template_variables_value import UpdateAgentRequestTemplateVariablesValueParams
    from .update_agent_request_tools_item import UpdateAgentRequestToolsItemParams
_dynamic_imports: typing.Dict[str, str] = {
    "AgentsCreateResponseParams": ".agents_create_response",
    "AgentsDeleteResponseParams": ".agents_delete_response",
    "AgentsGetResponseParams": ".agents_get_response",
    "AgentsListResponseParams": ".agents_list_response",
    "AgentsUpdateResponseParams": ".agents_update_response",
    "AgentsUpsertResponseParams": ".agents_upsert_response",
    "UpdateAgentRequestConfigurationEndpointParams": ".update_agent_request_configuration_endpoint",
    "UpdateAgentRequestTemplateVariablesValueParams": ".update_agent_request_template_variables_value",
    "UpdateAgentRequestToolsItemParams": ".update_agent_request_tools_item",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentsCreateResponseParams",
    "AgentsDeleteResponseParams",
    "AgentsGetResponseParams",
    "AgentsListResponseParams",
    "AgentsUpdateResponseParams",
    "AgentsUpsertResponseParams",
    "UpdateAgentRequestConfigurationEndpointParams",
    "UpdateAgentRequestTemplateVariablesValueParams",
    "UpdateAgentRequestToolsItemParams",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.extraction_schema import ExtractionSchemaParams


class ExtractionSchemasGetResponseParams(typing_extensions.TypedDict):
    extraction_schema: ExtractionSchemaParams



# This file was auto-generated by Fern from our API Definition.

import typing

from .basic_error import BasicError
from .validation_error import ValidationError

Error = typing.Union[BasicError, ValidationError]



# This file was auto-generated by Fern from our API Definition.

# isort: skip_file

import typing
from importlib import import_module

if typing.TYPE_CHECKING:
    from .agents_create_response import AgentsCreateResponse
    from .agents_delete_response import AgentsDeleteResponse
    from .agents_get_response import AgentsGetResponse
    from .agents_list_response import AgentsListResponse
    from .agents_update_response import AgentsUpdateResponse
    from .agents_upsert_response import AgentsUpsertResponse
    from .update_agent_request_audio_format import UpdateAgentRequestAudioFormat
    from .update_agent_request_background_noise import UpdateAgentRequestBackgroundNoise
    from .update_agent_request_configuration_endpoint import UpdateAgentRequestConfigurationEndpoint
    from .update_agent_request_template_variables_value import UpdateAgentRequestTemplateVariablesValue
    from .update_agent_request_tools_item import UpdateAgentRequestToolsItem
_dynamic_imports: typing.Dict[str, str] = {
    "AgentsCreateResponse": ".agents_create_response",
    "AgentsDeleteResponse": ".agents_delete_response",
    "AgentsGetResponse": ".agents_get_response",
    "AgentsListResponse": ".agents_list_response",
    "AgentsUpdateResponse": ".agents_update_response",
    "AgentsUpsertResponse": ".agents_upsert_response",
    "UpdateAgentRequestAudioFormat": ".update_agent_request_audio_format",
    "UpdateAgentRequestBackgroundNoise": ".update_agent_request_background_noise",
    "UpdateAgentRequestConfigurationEndpoint": ".update_agent_request_configuration_endpoint",
    "UpdateAgentRequestTemplateVariablesValue": ".update_agent_request_template_variables_value",
    "UpdateAgentRequestToolsItem": ".update_agent_request_tools_item",
}


def __getattr__(attr_name: str) -> typing.Any:
    module_name = _dynamic_imports.get(attr_name)
    if module_name is None:
        raise AttributeError(f"No {attr_name} found in _dynamic_imports for module name -> {__name__}")
    try:
        module = import_module(module_name, __package__)
        if module_name == f".{attr_name}":
            return module
        else:
            return getattr(module, attr_name)
    except ImportError as e:
        raise ImportError(f"Failed to import {attr_name} from {module_name}: {e}") from e
    except AttributeError as e:
        raise AttributeError(f"Failed to get {attr_name} from {module_name}: {e}") from e


def __dir__():
    lazy_attrs = list(_dynamic_imports.keys())
    return sorted(lazy_attrs)


__all__ = [
    "AgentsCreateResponse",
    "AgentsDeleteResponse",
    "AgentsGetResponse",
    "AgentsListResponse",
    "AgentsUpdateResponse",
    "AgentsUpsertResponse",
    "UpdateAgentRequestAudioFormat",
    "UpdateAgentRequestBackgroundNoise",
    "UpdateAgentRequestConfigurationEndpoint",
    "UpdateAgentRequestTemplateVariablesValue",
    "UpdateAgentRequestToolsItem",
]



# This file was auto-generated by Fern from our API Definition.

import typing_extensions
from ...requests.conversation import ConversationParams


class ConversationsGetResponseParams(typing_extensions.TypedDict):
    conversation: ConversationParams



# This file was auto-generated by Fern from our API Definition.

import typing_extensions


class ValidationErrorErrorParams(typing_extensions.TypedDict):
    message: str
    """
    Error message
    """
